WORDING =\
{
	'version_no': 'Python {}.{} not supported',
	'package_no': 'Package {} not found',
	'connection_no': 'Connection to {} not found',
	'daemon_no': 'Daemon for {} not found',
	'ip_no': 'IP for {} not found',
	'device_no': 'Device not found',
	'light_no': 'Light not found',
	'group_no': 'Group not found',
	'producer_no': 'Producer not found',
	'consumer_no': 'Consumer not found',
	'result_no': 'Result not found',
	'build_passed': 'Build of {} on {} passed',
	'build_process': 'Build of {} on {} in process',
	'build_errored': 'Build of {} on {} errored',
	'build_failed': 'Build of {} on {} failed',
	'setting_passed': 'Setting {} to build passed',
	'setting_process': 'Setting {} to build in process',
	'setting_errored': 'Setting {} to build errored',
	'setting_failed': 'Setting {} to build failed',
	'press_button': 'Press {} button on {}',
	'enable_feature': 'Enable {} on {}',
	'goodbye': 'Goodbye',
	'by': 'by',
	'hourglass': u'\u29d7',
	'tick': u'\u2714',
	'cross': u'\u2716',
	'point': '.',
	'comma': ',',
	'colon': ':',
	'question_mark': '?',
	'exclamation_mark': '!'
}


def get(key : str) -> str:
	return WORDING[key]



from typing import Any, Dict, List
from argparse import ArgumentParser
from chroma_feedback import helper, wording
from .api import get_api
from .group import get_groups, process_groups
from .light import get_lights, process_lights

ARGS = None


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--lifx-light-light', action = 'append')
		program.add_argument('--lifx-light-group', action = 'append')
	ARGS = helper.get_first(program.parse_known_args())


def run(status : str) -> List[Dict[str, Any]]:
	api = get_api()

	# use groups

	if ARGS.lifx_light_group:
		groups = get_groups(ARGS.lifx_light_group)

		if not groups:
			exit(wording.get('group_no') + wording.get('exclamation_mark'))
		return process_groups(groups, status)

	# use lights

	lights = get_lights(api.get_lights(), ARGS.lifx_light_light)

	if not lights:
		exit(wording.get('light_no') + wording.get('exclamation_mark'))
	return process_lights(lights, status)



from typing import Any, Dict, List
from chroma_feedback import color
from .api import get_api


def get_lights(ips : List[str]) -> List[Dict[str, Any]]:
	lights = []

	for ip in ips:
		lights.append(get_api(ip))
	return lights


def process_lights(lights : Any, status : str) -> List[Dict[str, Any]]:
	result = []

	# process lights

	for light in lights:
		light_name = light.get_properties()['name']

		if status == 'passed':
			result.append(
			{
				'consumer': 'xiaomi_yeelight',
				'type': 'light',
				'name': light_name,
				'active': static_light(light, color.get_passed()),
				'status': status
			})
		if status == 'process':
			result.append(
			{
				'consumer': 'xiaomi_yeelight',
				'type': 'light',
				'name': light_name,
				'active': static_light(light, color.get_process()),
				'status': status
			})
		if status == 'errored':
			result.append(
			{
				'consumer': 'xiaomi_yeelight',
				'type': 'light',
				'name': light_name,
				'active': static_light(light, color.get_errored()),
				'status': status
			})
		if status == 'failed':
			result.append(
			{
				'consumer': 'xiaomi_yeelight',
				'type': 'light',
				'name': light_name,
				'active': static_light(light, color.get_failed()),
				'status': status
			})
	return result


def static_light(light : Any, state : Dict[str, Any]) -> bool:
	return light.turn_on() == 'ok' and light.set_rgb(state['rgb'][0], state['rgb'][1], state['rgb'][2]) == 'ok'



from __future__ import print_function
from typing import Any
from argparse import ArgumentParser
import os
import sys
import threading
from chroma_feedback import consumer, helper, producer, reporter, wording


def run(program : ArgumentParser) -> None:
	if sys.version_info < (3, 4):
		exit(wording.get('version_no').format(sys.version_info.major, sys.version_info.minor) + wording.get('exclamation_mark'))

	# report header

	if threading.active_count() == 1:
		reporter.print_header()
		print()

	# process producer

	producer_result = producer.process(program)

	# handle exit

	if not producer_result:
		exit(wording.get('result_no') + wording.get('exclamation_mark'))

	# report producer

	producer_report = reporter.create_producer_report(producer_result)
	if producer_report:
		reporter.print_report(producer_report)
		print()

	# handle dry run

	args = helper.get_first(program.parse_known_args())
	if args.dry_run is False:

		# process consumer

		status = helper.get_producer_status(producer_result)
		consumer_result = consumer.process(program, status)

		# report consumer

		consumer_report = reporter.create_consumer_report(consumer_result)
		if consumer_report:
			reporter.print_report(consumer_report)
			print()

	# handle thread

	if args.background_run is True:
		threading.Timer(args.background_interval, run, args =
		[
			program
		]).start()


def destroy(signal_number : int, frame : Any) -> None:
	print('\r' + wording.get('goodbye') + wording.get('exclamation_mark'))
	os._exit(0)



from typing import Any, Dict, List
import copy
from chroma_feedback import color
from .api import get_api


def get_devices(devices : Any, device_names : List[str]) -> Any:
	if device_names:
		for device in copy.copy(devices):
			if device not in device_names:
				devices.remove(device)
	return devices


def process_devices(devices : Any, status : str) -> List[Dict[str, Any]]:
	result = []

	# process devices

	for device in devices:
		if status == 'passed':
			result.append(
			{
				'consumer': 'thingm_blink',
				'type': 'device',
				'name': device,
				'active': static_device(color.get_passed()),
				'status': status
			})
		if status == 'process':
			result.append(
			{
				'consumer': 'thingm_blink',
				'type': 'device',
				'name': device,
				'active': static_device(color.get_process()),
				'status': status
			})
		if status == 'errored':
			result.append(
			{
				'consumer': 'thingm_blink',
				'type': 'device',
				'name': device,
				'active': static_device(color.get_errored()),
				'status': status
			})
		if status == 'failed':
			result.append(
			{
				'consumer': 'thingm_blink',
				'type': 'device',
				'name': device,
				'active': static_device(color.get_failed()),
				'status': status
			})
	return result


def static_device(state : Dict[str, Any]) -> bool:
	api = get_api()

	return api is not None and api.fade_to_rgb(0, state['rgb'][0], state['rgb'][1], state['rgb'][2]) is None



from typing import Any, Dict, List
from argparse import ArgumentParser
import importlib
from chroma_feedback import helper, wording


def process(program : ArgumentParser) -> List[Dict[str, Any]]:
	args = helper.get_first(program.parse_known_args())
	result = []

	for producer_name in args.producer:
		producer = load_producer(producer_name)
		producer.init(program)
		result.extend(producer.run())
	return result


def load_producer(producer_name : str) -> Any:
	try:
		return importlib.import_module('chroma_feedback.producer.' + producer_name)
	except ImportError:
		exit(wording.get('producer_no') + wording.get('exclamation_mark'))



from typing import Any
from chroma_feedback import wording

API = None


def get_api(ip : str) -> Any:
	global API

	if not API:
		API = api_factory(ip)
	return API


def api_factory(ip : str) -> Any:
	api = None

	try:
		from yeelight import Bulb, BulbException

		try:
			api = Bulb(ip)
		except BulbException:
			exit(wording.get('connection_no').format('XIAOMI YEELIGHT') + wording.get('exclamation_mark'))
		try:
			api.turn_on()
		except BulbException:
			exit(wording.get('enable_feature').format('LAN CONTROL', 'XIAOMI YEELIGHT') + wording.get('exclamation_mark'))
		return api
	except ImportError:
		exit(wording.get('package_no').format('XIAOMI YEELIGHT') + wording.get('exclamation_mark'))



from typing import Any, Dict, List
from chroma_feedback import color
from .api import get_api, get_modes


def get_lights(ips : List[str]) -> List[Dict[str, Any]]:
	lights = []

	for ip in ips:
		lights.append(get_api(ip))
	return lights


def process_lights(lights : Any, status : str) -> List[Dict[str, Any]]:
	result = []

	# process lights

	for light in lights:

		if status == 'passed':
			result.append(
			{
				'consumer': 'magic_hue',
				'type': 'light',
				'name': light.name,
				'active': static_light(light, color.get_passed()),
				'status': status
			})
		if status == 'process':
			result.append(
			{
				'consumer': 'magic_hue',
				'type': 'light',
				'name': light.name,
				'active': static_light(light, color.get_process()),
				'status': status
			})
		if status == 'errored':
			result.append(
			{
				'consumer': 'magic_hue',
				'type': 'light',
				'name': light.name,
				'active': pulsate_light(light, color.get_errored()),
				'status': status
			})
		if status == 'failed':
			result.append(
			{
				'consumer': 'magic_hue',
				'type': 'light',
				'name': light.name,
				'active': pulsate_light(light, color.get_failed()),
				'status': status
			})
	return result


def static_light(light : Any, state : Dict[str, Any]) -> bool:
	modes = get_modes()

	if modes:
		light.mode = modes.CustomMode(
			mode = modes.MODE_GRADUALLY,
			speed = 1,
			colors = \
			[
				(state['rgb'][0], state['rgb'][1], state['rgb'][2]),
				(state['rgb'][0], state['rgb'][1], state['rgb'][2])
			]
		)
	return light.update_status() is None


def pulsate_light(light : Any, state : Dict[str, Any]) -> bool:
	modes = get_modes()

	if modes:
		light.mode = modes.CustomMode(
			mode = modes.MODE_GRADUALLY,
			speed = 1,
			colors = \
			[
				(state['rgb'][0], state['rgb'][1], state['rgb'][2]),
				(0, 0, 0)
			]
		)
	return light.update_status() is None



METADATA =\
{
	'name': 'chroma-feedback',
	'description': 'Turn your RGB powered hardware into an build indicator for continuous integration',
	'version': '7.5.1',
	'license': 'GPL-3.0',
	'keywords': 'appveyor bamboo circle codeship github gitlab jenkins teamcity travis',
	'author': 'Henry Ruhs',
	'author_email': 'info@redaxmedia.com',
	'url': 'https://github.com/redaxmedia/chroma-feedback'
}


def get(key : str) -> str:
	return METADATA[key]



from typing import Any, Dict, List
from argparse import ArgumentParser
import requests
from chroma_feedback import helper
from .normalize import normalize_data

ARGS = None


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--bamboo-host', required = True)
		program.add_argument('--bamboo-slug', action = 'append', required = True)
		program.add_argument('--bamboo-token', required = True)
	ARGS = helper.get_first(program.parse_known_args())


def run() -> List[Dict[str, Any]]:
	result = []

	for slug in ARGS.bamboo_slug:
		result.extend(fetch(ARGS.bamboo_host, slug, ARGS.bamboo_token))
	return result


def fetch(host : str, slug : str, token : str) -> List[Dict[str, Any]]:
	result = []
	response = None

	if host and slug and token:
		response = requests.get(host + '/rest/api/latest/result/' + get_slug(slug), headers =
		{
			'Accept': 'application/json',
			'Authorization': 'Bearer ' + token
		})

	# process response

	if response and response.status_code == 200:
		data = helper.parse_json(response)

		if 'results' in data and 'result' in data['results']:
			for plan in data['results']['result']:
				result.append(normalize_data(plan))
		elif data:
			result.append(normalize_data(data))
	return result


def get_slug(slug : str) -> str:
	if '-' in slug:
		return slug + '-latest'
	return slug



from typing import Any, Dict, List
from argparse import ArgumentParser
import requests
from chroma_feedback import helper
from .normalize import normalize_data

ARGS = None


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--wercker-host', default = 'https://app.wercker.com')
		program.add_argument('--wercker-slug', action = 'append', required = True)
		program.add_argument('--wercker-token', required = True)
	ARGS = helper.get_first(program.parse_known_args())


def run() -> List[Dict[str, Any]]:
	result = []

	for slug in ARGS.wercker_slug:
		result.extend(fetch(ARGS.wercker_host, slug, ARGS.wercker_token))
	return result


def fetch(host : str, slug : str, token : str) -> List[Dict[str, Any]]:
	result = []
	response = None

	if host and slug and token:
		response = requests.get(host + '/api/v3/applications/' + slug, headers =
		{
			'Bearer': token
		})

	# process response

	if response and response.status_code == 200:
		data = helper.parse_json(response)

		if 'id' in data:
			result.extend(fetch_runs(host, slug, data['id'], token))
	return result


def fetch_runs(host : str, slug : str, application_id : str, token : str) -> List[Dict[str, Any]]:
	result = []
	response = None

	if host and slug and application_id and token:
		response = requests.get(host + '/api/v3/runs?applicationId=' + application_id, headers =
		{
			'Bearer': token
		})

	# process response

	if response and response.status_code == 200:
		data = helper.parse_json(response)
		build = helper.get_first(data)

		if build:
			build['slug'] = slug
			result.append(normalize_data(build))
	return result



from typing import Any, Dict, List
from chroma_feedback import color
from .api import get_api


def get_groups(group_names : List[str]) -> List[Dict[str, Any]]:
	api = get_api()
	groups = []

	if group_names:
		for group_name in group_names:
			groups.append(api.get_devices_by_group(group_name))
	return groups


def get_group_name(group : Any) -> Any:
	for device in group.get_device_list():
		return device.get_group_label()


def process_groups(groups : Any, status : str) -> List[Dict[str, Any]]:
	result = []

	# process groups

	for group in groups:
		group_name = get_group_name(group)

		if status == 'passed':
			result.append(
			{
				'consumer': 'lifx_light',
				'type': 'group',
				'name': group_name,
				'active': static_group(group, color.get_passed()),
				'status': status
			})
		if status == 'process':
			result.append(
			{
				'consumer': 'lifx_light',
				'type': 'group',
				'name': group_name,
				'active': static_group(group, color.get_process()),
				'status': status
			})
		if status == 'errored':
			result.append(
			{
				'consumer': 'lifx_light',
				'type': 'group',
				'name': group_name,
				'active': static_group(group, color.get_errored()),
				'status': status
			})
		if status == 'failed':
			result.append(
			{
				'consumer': 'lifx_light',
				'type': 'group',
				'name': group_name,
				'active': static_group(group, color.get_failed()),
				'status': status
			})
	return result


def static_group(group : Any, state : Dict[str, Any]) -> bool:
	return group.set_power('on') is None and group.set_color(
	[
		state['hue'],
		state['saturation'][2],
		state['brightness'][2],
		state['kelvin']
	]) is None



from typing import Any, Dict

COLOR =\
{
	'red': '\033[0;31m',
	'green': '\033[0;32m',
	'yellow': '\033[0;33m',
	'end': '\033[0m'
}


def format_red(text : str) -> str:
	return COLOR['red'] + text + COLOR['end']


def format_green(text : str) -> str:
	return COLOR['green'] + text + COLOR['end']


def format_yellow(text : str) -> str:
	return COLOR['yellow'] + text + COLOR['end']


def get_passed() -> Dict[str, Any]:
	return\
	{
		'rgb':
		[
			0,
			255,
			0
		],
		'hue': 26000,
		'saturation':
		[
			100,
			255,
			65535
		],
		'brightness':
		[
			100,
			255,
			65535
		],
		'kelvin': 3500
	}


def get_process() -> Dict[str, Any]:
	return\
	{
		'rgb':
		[
			255,
			255,
			0
		],
		'hue': 10000,
		'saturation':
		[
			100,
			255,
			65535
		],
		'brightness':
		[
			100,
			255,
			65535
		],
		'kelvin': 3500
	}


def get_errored() -> Dict[str, Any]:
	return\
	{
		'rgb':
		[
			255,
			255,
			255
		],
		'hue': 58000,
		'saturation':
		[
			0,
			0,
			0
		],
		'brightness':
		[
			100,
			255,
			65535
		],
		'kelvin': 3500
	}


def get_failed() -> Dict[str, Any]:
	return\
	{
		'rgb':
		[
			255,
			0,
			0
		],
		'hue': 0,
		'saturation':
		[
			100,
			255,
			65535
		],
		'brightness':
		[
			100,
			255,
			65535
		],
		'kelvin': 3500
	}



from typing import Any, Dict, List
from argparse import ArgumentParser
import base64
import requests
from chroma_feedback import helper
from .normalize import normalize_data

ARGS = None


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--codeship-host', default = 'https://api.codeship.com')
		program.add_argument('--codeship-slug', action = 'append')
		program.add_argument('--codeship-username', required = True)
		program.add_argument('--codeship-password', required = True)
	ARGS = helper.get_first(program.parse_known_args())


def run() -> List[Dict[str, Any]]:
	result = []
	auth = fetch_auth(ARGS.codeship_host, ARGS.codeship_username, ARGS.codeship_password)

	for organization in auth['organizations']:
		result.extend(fetch(ARGS.codeship_host, organization['uuid'], ARGS.codeship_slug, auth['token']))
	return result


def fetch(host : str, organization : str, slug : str, token : str) -> List[Dict[str, Any]]:
	result = []
	response = None

	if host and organization and token:
		response = requests.get(host + '/v2/organizations/' + organization + '/projects', headers =
		{
			'Authorization': 'Bearer ' + token
		})

	# process response

	if response and response.status_code == 200:
		data = helper.parse_json(response)

		for project in data['projects']:
			project_id = str(project['id'])
			if not slug or project_id in slug:
				result.extend(fetch_builds(host, organization, project['uuid'], token))
	return result


def fetch_builds(host : str, organization : str, project : str, token : str) -> List[Dict[str, Any]]:
	result = []
	response = None

	if host and organization and project and token:
		response = requests.get(host + '/v2/organizations/' + organization + '/projects/' + project + '/builds', headers =
		{
			'Authorization': 'Bearer ' + token
		})

	# process response

	if response and response.status_code == 200:
		data = helper.parse_json(response)

		if 'builds' in data:
			build = helper.get_first(data['builds'])
			if build:
				result.append(normalize_data(build))
	return result


def fetch_auth(host : str, username : str, password : str) -> Dict[str, Any]:
	result = {}
	response = None

	if host and username and password:
		username_token = username + ':' + password
		response = requests.post(host + '/v2/auth', headers =
		{
			'Authorization': 'Basic ' + base64.b64encode(username_token.encode('utf-8')).decode('ascii')
		})

	# process response

	if response and response.status_code == 200:
		data = helper.parse_json(response)

		if 'access_token' and 'organizations' in data:
			result['token'] = data['access_token']
			result['organizations'] = data['organizations']
	return result



from typing import Any, Dict, List
from argparse import ArgumentParser
import base64
import requests
from chroma_feedback import helper
from .normalize import normalize_data

ARGS = None


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--jenkins-host', required = True)
		program.add_argument('--jenkins-slug', action = 'append', required = True)
		program.add_argument('--jenkins-username')
		program.add_argument('--jenkins-password')
	ARGS = helper.get_first(program.parse_known_args())


def run() -> List[Dict[str, Any]]:
	result = []

	for slug in ARGS.jenkins_slug:
		result.extend(fetch(ARGS.jenkins_host, slug, ARGS.jenkins_username, ARGS.jenkins_password))
	return result


def fetch(host : str, slug : str, username : str, password : str) -> List[Dict[str, Any]]:
	result = []
	response = None

	if host and slug and username and password:
		username_password = username + ':' + password
		response = requests.get(host + '/job/' + slug + '/lastBuild/api/json', headers =
		{
			'Authorization': 'Basic ' + base64.b64encode(username_password.encode('utf-8')).decode('ascii')
		})
	elif host and slug:
		response = requests.get(host + '/job/' + slug + '/lastBuild/api/json')

	# process response

	if response and response.status_code == 200:
		data = helper.parse_json(response)

		if data:
			data['slug'] = slug
			result.append(normalize_data(data))
	return result
